// Code generated by ent, DO NOT EDIT.

package ent

import (
	"context"
	"errors"
	"fmt"

	"entgo.io/ent/dialect/sql/sqlgraph"
	"entgo.io/ent/schema/field"
	"github.com/google/uuid"
	"github.com/in-toto/archivista/ent/attestation"
	"github.com/in-toto/archivista/ent/attestationcollection"
)

// AttestationCreate is the builder for creating a Attestation entity.
type AttestationCreate struct {
	config
	mutation *AttestationMutation
	hooks    []Hook
}

// SetType sets the "type" field.
func (ac *AttestationCreate) SetType(s string) *AttestationCreate {
	ac.mutation.SetType(s)
	return ac
}

// SetID sets the "id" field.
func (ac *AttestationCreate) SetID(u uuid.UUID) *AttestationCreate {
	ac.mutation.SetID(u)
	return ac
}

// SetNillableID sets the "id" field if the given value is not nil.
func (ac *AttestationCreate) SetNillableID(u *uuid.UUID) *AttestationCreate {
	if u != nil {
		ac.SetID(*u)
	}
	return ac
}

// SetAttestationCollectionID sets the "attestation_collection" edge to the AttestationCollection entity by ID.
func (ac *AttestationCreate) SetAttestationCollectionID(id uuid.UUID) *AttestationCreate {
	ac.mutation.SetAttestationCollectionID(id)
	return ac
}

// SetAttestationCollection sets the "attestation_collection" edge to the AttestationCollection entity.
func (ac *AttestationCreate) SetAttestationCollection(a *AttestationCollection) *AttestationCreate {
	return ac.SetAttestationCollectionID(a.ID)
}

// Mutation returns the AttestationMutation object of the builder.
func (ac *AttestationCreate) Mutation() *AttestationMutation {
	return ac.mutation
}

// Save creates the Attestation in the database.
func (ac *AttestationCreate) Save(ctx context.Context) (*Attestation, error) {
	ac.defaults()
	return withHooks(ctx, ac.sqlSave, ac.mutation, ac.hooks)
}

// SaveX calls Save and panics if Save returns an error.
func (ac *AttestationCreate) SaveX(ctx context.Context) *Attestation {
	v, err := ac.Save(ctx)
	if err != nil {
		panic(err)
	}
	return v
}

// Exec executes the query.
func (ac *AttestationCreate) Exec(ctx context.Context) error {
	_, err := ac.Save(ctx)
	return err
}

// ExecX is like Exec, but panics if an error occurs.
func (ac *AttestationCreate) ExecX(ctx context.Context) {
	if err := ac.Exec(ctx); err != nil {
		panic(err)
	}
}

// defaults sets the default values of the builder before save.
func (ac *AttestationCreate) defaults() {
	if _, ok := ac.mutation.ID(); !ok {
		v := attestation.DefaultID()
		ac.mutation.SetID(v)
	}
}

// check runs all checks and user-defined validators on the builder.
func (ac *AttestationCreate) check() error {
	if _, ok := ac.mutation.GetType(); !ok {
		return &ValidationError{Name: "type", err: errors.New(`ent: missing required field "Attestation.type"`)}
	}
	if v, ok := ac.mutation.GetType(); ok {
		if err := attestation.TypeValidator(v); err != nil {
			return &ValidationError{Name: "type", err: fmt.Errorf(`ent: validator failed for field "Attestation.type": %w`, err)}
		}
	}
	if _, ok := ac.mutation.AttestationCollectionID(); !ok {
		return &ValidationError{Name: "attestation_collection", err: errors.New(`ent: missing required edge "Attestation.attestation_collection"`)}
	}
	return nil
}

func (ac *AttestationCreate) sqlSave(ctx context.Context) (*Attestation, error) {
	if err := ac.check(); err != nil {
		return nil, err
	}
	_node, _spec := ac.createSpec()
	if err := sqlgraph.CreateNode(ctx, ac.driver, _spec); err != nil {
		if sqlgraph.IsConstraintError(err) {
			err = &ConstraintError{msg: err.Error(), wrap: err}
		}
		return nil, err
	}
	if _spec.ID.Value != nil {
		if id, ok := _spec.ID.Value.(*uuid.UUID); ok {
			_node.ID = *id
		} else if err := _node.ID.Scan(_spec.ID.Value); err != nil {
			return nil, err
		}
	}
	ac.mutation.id = &_node.ID
	ac.mutation.done = true
	return _node, nil
}

func (ac *AttestationCreate) createSpec() (*Attestation, *sqlgraph.CreateSpec) {
	var (
		_node = &Attestation{config: ac.config}
		_spec = sqlgraph.NewCreateSpec(attestation.Table, sqlgraph.NewFieldSpec(attestation.FieldID, field.TypeUUID))
	)
	if id, ok := ac.mutation.ID(); ok {
		_node.ID = id
		_spec.ID.Value = &id
	}
	if value, ok := ac.mutation.GetType(); ok {
		_spec.SetField(attestation.FieldType, field.TypeString, value)
		_node.Type = value
	}
	if nodes := ac.mutation.AttestationCollectionIDs(); len(nodes) > 0 {
		edge := &sqlgraph.EdgeSpec{
			Rel:     sqlgraph.M2O,
			Inverse: true,
			Table:   attestation.AttestationCollectionTable,
			Columns: []string{attestation.AttestationCollectionColumn},
			Bidi:    false,
			Target: &sqlgraph.EdgeTarget{
				IDSpec: sqlgraph.NewFieldSpec(attestationcollection.FieldID, field.TypeUUID),
			},
		}
		for _, k := range nodes {
			edge.Target.Nodes = append(edge.Target.Nodes, k)
		}
		_node.attestation_collection_attestations = &nodes[0]
		_spec.Edges = append(_spec.Edges, edge)
	}
	return _node, _spec
}

// AttestationCreateBulk is the builder for creating many Attestation entities in bulk.
type AttestationCreateBulk struct {
	config
	err      error
	builders []*AttestationCreate
}

// Save creates the Attestation entities in the database.
func (acb *AttestationCreateBulk) Save(ctx context.Context) ([]*Attestation, error) {
	if acb.err != nil {
		return nil, acb.err
	}
	specs := make([]*sqlgraph.CreateSpec, len(acb.builders))
	nodes := make([]*Attestation, len(acb.builders))
	mutators := make([]Mutator, len(acb.builders))
	for i := range acb.builders {
		func(i int, root context.Context) {
			builder := acb.builders[i]
			builder.defaults()
			var mut Mutator = MutateFunc(func(ctx context.Context, m Mutation) (Value, error) {
				mutation, ok := m.(*AttestationMutation)
				if !ok {
					return nil, fmt.Errorf("unexpected mutation type %T", m)
				}
				if err := builder.check(); err != nil {
					return nil, err
				}
				builder.mutation = mutation
				var err error
				nodes[i], specs[i] = builder.createSpec()
				if i < len(mutators)-1 {
					_, err = mutators[i+1].Mutate(root, acb.builders[i+1].mutation)
				} else {
					spec := &sqlgraph.BatchCreateSpec{Nodes: specs}
					// Invoke the actual operation on the latest mutation in the chain.
					if err = sqlgraph.BatchCreate(ctx, acb.driver, spec); err != nil {
						if sqlgraph.IsConstraintError(err) {
							err = &ConstraintError{msg: err.Error(), wrap: err}
						}
					}
				}
				if err != nil {
					return nil, err
				}
				mutation.id = &nodes[i].ID
				mutation.done = true
				return nodes[i], nil
			})
			for i := len(builder.hooks) - 1; i >= 0; i-- {
				mut = builder.hooks[i](mut)
			}
			mutators[i] = mut
		}(i, ctx)
	}
	if len(mutators) > 0 {
		if _, err := mutators[0].Mutate(ctx, acb.builders[0].mutation); err != nil {
			return nil, err
		}
	}
	return nodes, nil
}

// SaveX is like Save, but panics if an error occurs.
func (acb *AttestationCreateBulk) SaveX(ctx context.Context) []*Attestation {
	v, err := acb.Save(ctx)
	if err != nil {
		panic(err)
	}
	return v
}

// Exec executes the query.
func (acb *AttestationCreateBulk) Exec(ctx context.Context) error {
	_, err := acb.Save(ctx)
	return err
}

// ExecX is like Exec, but panics if an error occurs.
func (acb *AttestationCreateBulk) ExecX(ctx context.Context) {
	if err := acb.Exec(ctx); err != nil {
		panic(err)
	}
}
